/*
 * BaseShip.h
 *
 * Defines a ship and all it's characteristics as defined by the
 * specification file.
 *
 * (C) 2001 MagnumMan-iCoP (James E. King, III)
 */

#include "Race.h"					// From the SFC2API
#include "eshiptypes.h"
#include "EnhancedShiplist.h"

#include <string>					// From MSVC++
#include <map>						

#include "WeaponMount.h"			// From this library

typedef std::map< eSystemHardPoint, WeaponMount >	Weapons;
typedef Weapons::iterator							WeaponsIter;

class BaseShip
{
public:		// could make private with accessors, but why? :>

	// Each of these fields corresponds to a column in the shiplist file

	// General Characteristics

	eRace			race;			// Race.h
	eHullType		hullType;		// eshiptypes.h
	std::string		shipCode;		// "F-FQL" for example
	eClassTypes		shipClass;		// shipclasses.h (included in eshiptypes.h) 
	int32			bpv;			// The base BPV defined in the spec file
	eSpecialRole	specialRole;	// eshiptypes.h

	int32			yearFirstAvail;	// In SFC years
	int32			yearLastAvail;	// In SFC years

	enum
	{
		kEarlyEraBegin       = -999,
		kEarlyEraEnd         = 9,
		kMiddleEraBegin      = 10,
		kMiddleEraEnd        = 19,
		kLateEraBegin        = 20,
		kLateEraEnd          = 36,
		kAdvancedEraBegin    = 37,
		kAdvancedEraEnd      = 999,
		kNeverDecommissioned = 999	// used in yearLastAvailable
	};

	// Movement and Size

	int32			sizeClass;		// 1 through 5
	eTurnMode		turnMode;		// eshiptypes.h
	float			moveCost;		// 0.00 through 2.00
	int32			hetPlusTwo;		// no idea what this value (0..2) does
	int32			hetBreakdown;	// have not determined number to ship relation
	int32			stealthECM;		// Amount of hidden ECM

	// Personnel

	int32			regularCrew;
	int32			marinesBase;	// Standard marine complementary
	int32			marinesMax;
	int32			deckCrew;		// difference between this and regular crew?
	int32			totalCrew;		// This is probably the one anybody cares about
	int32			minimumCrew;

	// Shielding

	int32			shields[kShieldIndexCount];
	int32			totalShielding;

	// Cloak
	
	int32			cloakCost;		// energy cost for cloak

	enum
	{
		kNoCloak = 0				// for cloakCost
	};

	// Weaponry
	Weapons			weapons;		// ex: weapons[kHeavyWeapon2HardPoint]
									//			will return (WeaponMount &)
	int32			probes;			// Spec file has 1 for every 5, we list 5 or 10
	int32			tBombsBase;
	int32			tBombsMax;
	int32			nukeBombsBase;
	int32			nukeBombsMax;
	int32			maxDroneControl;
	int32			add_6;			// extinct; use hardpoints instead - numbers not accurate
	int32			add_12;			// extinct; use hardpoints instead - numbers not accurate

	// Shuttles & Fighters

	int32			shuttlesSize;	// Unsure what this means
	int32			launchRate;		// Launches per turn for shuttles
	int32			adminBase;		// Base number of admin shuttles
	int32			adminMax;

	// Hull & Armor

	int32			armor;
	int32			forwardHull;
	int32			centerHull;
	int32			aftHull;
	int32			cargo;
	int32			barracks;
	int32			repair;			// This is not the number of standard repairs
	int32			leftWarp;
	int32			centerWarp;
	int32			rightWarp;
	int32			impulse;
	int32			apr;
	int32			battery;
	int32			bridge;
	int32			security;
	int32			lab;
	int32			transporters;	// This is accurate
	int32			tractors;
	int32			mechTractors;
	int32			specialSensors;
	int32			sensors;
	int32			scanners;
	int32			explosionStrength;
	int32			acceleration;
	int32			damageControl;
	int32			extraDamage;
	int32			shipCost;		// all 5, don't know what this is

	std::string		baseRefitClass;	// such as "F-FFE" for a "F-FFE+" entry
	std::string		geometry;
	std::string		geoUI;
	std::string		fullName;		// "Guided Weapons Destroyer" for example
	std::string		refits;

	int32			balance;		// individual ship BPV adjustment for balancing

	// Fighter specific items

	int32			speed;
	int32			stealthECCM;	// Amount of hidden ECCM
	int32			damage;
	int32			groundAttackBonus;
	int32			carrierSizeClass;


	// Enhanced shiplist items -- FS
	eEnhancedClassTypes enhancedShipClass;
	eEnhancedRoles enhancedRole;
	eEnhancedRoles enhancedSecondaryRole;
	eEnhancedXTech enhancedXtech;
	eEnhancedProductionAvailability enhancedProductionAvailability;

//////////////////////////////////////////////////////////////////////////

	BaseShip() { }
	~BaseShip() { }

};

/*
 * 20020121 JEK
 * Some definitions are required to fill in for some missing enums
 * in the SFC OP API
 */

#define kSpecialRoleE		(eSpecialRole)(kNumberOfSpecialRoles + 1)
#define kSpecialRoleI		(eSpecialRole)(kNumberOfSpecialRoles + 2)
#define kSpecialRoleL		(eSpecialRole)(kNumberOfSpecialRoles + 3)
#define kSpecialRoleP		(eSpecialRole)(kNumberOfSpecialRoles + 4)
#define kSpecialRoleQ		(eSpecialRole)(kNumberOfSpecialRoles + 5)
#define kSpecialRoleT		(eSpecialRole)(kNumberOfSpecialRoles + 6)
#define kSpecialRoleV		(eSpecialRole)(kNumberOfSpecialRoles + 7)
#define kSpecialRolePuck	(eSpecialRole)(kNumberOfSpecialRoles + 8)
#define kSpecialRoleA		(eSpecialRole)(kNumberOfSpecialRoles + 9)

#define kWeaponArcFXL		(eWeaponArc)(kNumberOfWeaponArcs + 1)
#define kWeaponArcFXR		(eWeaponArc)(kNumberOfWeaponArcs + 2)
#define kWeaponArcRXL		(eWeaponArc)(kNumberOfWeaponArcs + 3)
#define kWeaponArcRXR		(eWeaponArc)(kNumberOfWeaponArcs + 4)
