/*
 * ShipList.h
 *
 * Defines a group of ships.
 *
 * (C) 2001 MagnumMan-iCoP (James E. King, III)
 */

#include "Ship.h"
#include "EnhancedShipList.h"
#include <list>

// For EAW/OP detection
#include "soundnums.h"
#ifdef kSndPhaserFire
	#define EAW 1
#else
	#define OP 1
#endif



typedef std::list< Ship >		RealShipList;
typedef RealShipList::iterator	RealShipListIter;

#define SHIP_LIST_ITER(sl) \
	for (RealShipListIter iter = (sl).begin(); iter != (sl).end(); iter++) \
	{
		//
		// In here you can do things like...
		//
		// printf("Ship Code: %s\n", iter->shipCode.c_str());
		//
#define SHIP_LIST_ITER_END \
	}

//
// On a Pentium III 800MHz, it takes roughly 590ms on average to
// load the ship list using CreateMaster.
//

//
// Since the ShipList is a subclass of RealShipList (a std::list
// where the Ships are stored), you can use a RealShipListIter
// as an iterator and process the ships like this:
//
// ShipList sl;
// if (!sl.CreateMaster(ShipList::kEmpiresAtWar, false)) throw errCode;
// SHIP_LIST_ITER(sl)
//    Ship s = *iter;
//    ...
// SHIP_LIST_ITER_END
//
// For a good example if you have the source code to ShipList you
// can look at Find.cpp which uses the iterator extensively.
//

//
// After you create a master ShipList, keep it safe in memory and work
// off copies.  You can copy a ShipList normally using the = operator.
// For example:
//
// ShipList sl;
// if (!sl.CreateMaster(ShipList::kOrionPirates, false)) throw errCode;
// ShipList fedList = sl;
//   // use fedList.subset to restrict list to Federation
//

class ShipList : public RealShipList
{
private:
	RealShipList fighters_and_pfs;		// Never manipulated by Subset,
											// required for proper creation of
											// ship or fleet.


public:
	ShipList() { }
	~ShipList() { }

	typedef enum
	{
		kEmpiresAtWar,
		kOrionPirates
	} eGame;
	
	//
	// CreateFighterList
	//
	// Reads the ship list from the specification files
	// Since the list is made entirely from classes and std:: items,
	// it will self destruct when it pops off the stack (unless you
	// allocate the class with new, in which case it's in the heap)
	//
	// The first variant of the call will look in the registry to find
	// the proper location of the shiplist and ftrlist files.  This
	// obviously assumes you have the game installed.
	//
	// Params:	game		Select the game files you want to work with
	//			meta		If true, the MetaAssets are read, otherwise the regular
	//						assets are read.
	//          filename	Load a specific file comtaining the ship list
	//
	// Returns:	bool		Indicating success or failure
	//

	bool	CreateFighterList(eGame game, bool meta);
	bool	CreateFighterList(const char *specname, const char *ftrname);
	bool	CreateFighterList(std::string specname, std::string ftrname);

	//
	// CreateMaster
	//
	// Reads the ship list from the specification files
	// Since the list is made entirely from classes and std:: items,
	// it will self destruct when it pops off the stack (unless you
	// allocate the class with new, in which case it's in the heap)
	//
	// The first variant of the call will look in the registry to find
	// the proper location of the shiplist and ftrlist files.  This
	// obviously assumes you have the game installed.
	//
	// Params:	game		Select the game files you want to work with
	//			meta		If true, the MetaAssets are read, otherwise the regular
	//						assets are read.
	//          filename	Load a specific file comtaining the ship list
	//
	// Returns:	bool		Indicating success or failure
	//

	bool	CreateMaster(eGame game, bool meta);
	bool	CreateMaster(const char *specname, const char *ftrname);
	bool	CreateMaster(std::string specname, std::string ftrname);

	//
	// Find
	//
	// Locates the first ship matching given characteristics.
	// It returns true if it finds a ship and false if it does not.
	// If it returns true, aShip is initialized, otherwise it is not.
	//

	bool	Find(std::string shipCode, Ship &aShip);		// "F-FQL" for example
	bool	Find(const char *shipCode, Ship &aShip);
	bool	FindFighter(std::string shipCode, Ship &aShip);
	bool	FindFighter(const char *shipCode, Ship &aShip);

	// Returns fighters and PFs list for parsing.
	RealShipList ShipList::FighterList( void );

	//
	// Remove
	//
	// Allows you to remove a ship from the list.
	// If the ship is found and removed, the function returns true
	//

	bool	Remove(std::string shipCode);					// "F-BCF" for example
	bool	Remove(const char *shipCode);


	//
	// Subset
	//
	// These routines make a subset out of the current ship list.
	// Use these functions to narrow down a selection set.
	//
	// You can choose to include or exclude the set that you pass
	// in the first parameter.  For example:
	//
	// std::list< eRace >	raceLimit;
	// raceLimit.push_back(kFederation);
	// raceLimit.push_back(kLyran);
	//
	// Subset(raceLimit)
	//		will pare down the list to Federation and Lyran
	//
	// Subset(raceLimit, false)
	//		will omit Federation and Lyran from the current list
	//
	// Why use std::list?  Well, because the constants in the API
	// are not all bitmaps, and therefore need to be treated either
	// like a range (i.e. "kFederation through kLyran") or with
	// an inclusion/exclusion list like the one I decided to use.
	//

	int32	Subset(std::list< eRace > &,				bool include = true);
	int32	Subset(std::list< eHullType > &,			bool include = true);
	int32	Subset(std::list< eClassTypes > &,			bool include = true);
	int32	Subset(std::list< eTurnMode >&,				bool include = true);

	// enhanced SLAPI subsets --FS
	int32	Subset(std::list< eEnhancedClassTypes > &,	bool include = true);
	int32   Subset(std::list< eEnhancedRoles > &,		bool include = true);
	int32   Subset(std::list< eEnhancedXTech > &,		bool include = true);
	int32	Subset(std::list< eEnhancedProductionAvailability > &, bool include = true);


	// a faster way to devide everything for class hull subsets --FS
	int32	mHullClassSubsets(
		ShipList *mon_sl,
		ShipList *bb_sl,
		ShipList *dn_sl,
		ShipList *cv_sl,
		ShipList *bch_sl,
		ShipList *nca_sl,
		ShipList *ca_sl,
		ShipList *cl_sl,
		ShipList *wd_sl,
		ShipList *dd_sl,
		ShipList *ff_sl
	);

	int32 mEnhancedHullClassSubsets(
		ShipList *mon_sl,
		ShipList *bb_sl,
		ShipList *dnh_sl,
		ShipList *dn_sl,
		ShipList *monitor_sl,
		ShipList *bch_sl,
		ShipList *nca_sl,
		ShipList *ca_sl,
		ShipList *cm_sl,
		ShipList *hdw_sl,
		ShipList *cw_sl,
		ShipList *cl_sl,
		ShipList *dw_sl,
		ShipList *dd_sl,
		ShipList *ff_sl,
		ShipList *ntg_sl,
		ShipList *tug_sl,
		ShipList *ltg_sl,
		ShipList *axh_sl,
		ShipList *axl_sl,
		ShipList *axs_sl
	);

	typedef enum
	{
		kSubsetBPV,
		kSubsetSizeClass,
		kSubsetYearAvail,
		kSubsetYearFirstAvail, // -- FS
		kSubsetYearLastAvail, // -- FS
		kSubsetYearRangeExists, // --FS
	} IntegerSubset;
	
	int32	Subset(IntegerSubset,  int32 min, int32 max, bool include = true);

	typedef enum
	{
		kMoveCost
	} FloatingSubset;

	int32	Subset(FloatingSubset, int32 min, int32 max, bool include = true);

	//
	// SelectRandomShip
	//
	// Selects a random ship from the current list.  Useful if you have
	// subsetted the list down, or if you want a real random fight.
	// It returns true if it finds a ship and false if it does not.
	// If it returns true, aShip is initialized, otherwise it is not.
	//

	bool	SelectRandomShip(Ship &);

	//
	// CreateFleet
	//
	// Uses the input parameters to create a fleet, which is returned in the
	// resulting ShipList.
	// 
	// Admittedly, this function only works off the base BPV of a ship and
	// does not attempt to modify the loadout in any way right now, including
	// fighters.
	//

	ShipList CreateFleet(int32 totalShips, int32 totalBPV);

	//
	// MinimumBPV
	// MaximumBPV
	// TotalBPV
	//
	// Returns the (minimum, maximum, total) bpv of all ships in the list.
	//

	int32	MinimumBPV(void);
	int32	MaximumBPV(void);
	int32	TotalBPV(void);

	//
	// SubsetPlayableRaces
	// SubsetPlayableShipClasses
	//
	// These inline functions just make it easier to select
	// the 8 major races or to select the list of playable
	// ship classes.
	//

	inline int32 SubsetPlayableRaces(bool includeOrions = false)
	{
		std::list< eRace >	races;
		races.push_back(kFederation);
		races.push_back(kKlingon);
		races.push_back(kRomulan);
		races.push_back(kGorn);
		races.push_back(kLyran);
		races.push_back(kHydran);
		races.push_back(kISC);
		races.push_back(kMirak);

#ifdef OP
		if (includeOrions)
		{
			races.push_back(kOrionOrion);
			races.push_back(kOrionKorgath);
			races.push_back(kOrionPrime);
			races.push_back(kOrionTigerHeart);
			races.push_back(kOrionBeastRaiders);
			races.push_back(kOrionSyndicate);
			races.push_back(kOrionWyldeFire);
			races.push_back(kOrionCamboro);
		}
#endif

		return Subset(races);
	}

	inline int32 SubsetPlayableShipClasses(void)
	{
		std::list< eClassTypes >	classes;
		classes.push_back(kClassFrigate);
		classes.push_back(kClassDestroyer);
		classes.push_back(kClassWarDestroyer);
		classes.push_back(kClassLightCruiser);
		classes.push_back(kClassHeavyCruiser);
		classes.push_back(kClassNewHeavyCruiser);
		classes.push_back(kClassHeavyBattlecruiser);
		classes.push_back(kClassCarrier);
		classes.push_back(kClassDreadnought);
		classes.push_back(kClassBattleship);
		return Subset(classes);
	}

	//
	// Special utility classes for easy conversions
	//
	
	eRace			decodeRace(const char *raceStr);
	eHullType		decodeHullType(const char *hullStr);
	eClassTypes		decodeShipClass(const char *classStr);
	eSpecialRole	decodeSpecialRole(const char *roleStr);
	eTurnMode		decodeTurnMode(const char *turnMode);
	eShipSystemType decodeWeaponType(const char *weaponStr, bool isFighter = false);
	eWeaponArc		decodeWeaponArc(const char *arcStr);

	// enhanced SL API entries
	eEnhancedClassTypes	decodeEnhancedShipClass(const char *classStr);
	eEnhancedRoles decodeEnhancedRole(const char *roleStr);
	eEnhancedXTech decodeEnhancedXTech(const char *xtechStr);
	eEnhancedProductionAvailability decodeProductionAvailability(const char *availStr);

	bool	mIsEnhancedShiplist(void);

private:
	bool			FindSpecsDirectory(eGame game, bool meta, std::string &location);
	bool			decodeShip(const char *data, Ship &aShip, bool pfRun = false);
	bool			decodeFighter(const char *data, Ship &aShip);
};


